'''
This is the main implementation file for clastline
'''
import contextlib
import os
import shutil
import sys


class cLastLine:
    '''
    cLastLine is a class that will manage the final line of terminal output
    '''
    def __init__(self, stream=sys.stdout, lineEnd=os.linesep):
        '''
        Initializer. This takes in the stream to write to, along with lineEnd, which is the character to add at the end of the last line upon exiting the context manager.
        '''
        self._stream = sys.stdout
        self._lineEnd = lineEnd
        self._writtenData = ''

    __enter__ = lambda self: self
    __exit__ = lambda self, *args, **kwargs: self.close()

    def close(self):
        '''
        will add a lineEnd to the stream if needed.
        Note: Will NOT close the underlying stream.
        '''
        if self._writtenData and (self._writtenData[-1] != self._lineEnd):
            self.write(self._lineEnd, clearBeforeWrite=False)

    def clearLine(self):
        '''
        If there has been data already written, writes spaces to effectively clear the line
        '''
        if self._writtenData != '':
            self.write(shutil.get_terminal_size().columns * ' ', clearBeforeWrite=False)

    def write(self, txt, clearBeforeWrite=True):
        '''
        Main user facing function. Writes the given text over the text on the last line of the terminal.

        If clearBeforeWrite is set to True, will call clearLine() before writing, otherwise will not.

        The stream will be automatically flushed immediately after it is written.
        '''
        if clearBeforeWrite:
            self.clearLine()

        if self._writtenData != '':
            txt = '\r' + txt

        self._writtenData += txt
        self._stream.write(txt)
        self._stream.flush()



import os
import shutil
import unittest.mock

from .clastline import cLastLine


def test_simple():
    with cLastLine() as cll:
        cll.write('hi')
        cll.write('word')
        cll.write('test', clearBeforeWrite=False)

    clearText = shutil.get_terminal_size().columns * ' '
    assert cll._writtenData == f'''hi\r{clearText}\rword\rtest\r{os.linesep}'''

def test_nothing_written_on_nothing_written():
    with cLastLine() as cll:
        pass

    assert cll._writtenData == ''

def test_one_line_with_alt_end():
    with cLastLine(lineEnd='bleh') as cll:
        cll.write('a')

    assert cll._writtenData == 'a\rbleh'



from setuptools import setup
import os
import sys

sys.path.insert(0, os.path.join(os.path.dirname(__file__), 'clastline'))
from __init__ import __version__

setup(
    name='clastline',
    author='csm10495',
    author_email='csm10495@gmail.com',
    url='http://github.com/csm10495/clastline',
    version=__version__,
    packages=['clastline'],
    license='MIT License',
    python_requires='>=3.6',
    long_description=open(os.path.join(os.path.dirname(__file__), 'README.md')).read(),
    long_description_content_type='text/markdown',
    classifiers=[
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Operating System :: Microsoft :: Windows',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
    ],
    include_package_data = True,
    install_requires=[''],
    entry_points={},
)